==> analytics.js-integration-nielsen-dcr-0.0.3.tgz: package/lib/index.js <==
'use strict';

/**
 * Module dependencies.
 */

var integration = require('@segment/analytics.js-integration');
var useHttps = require('use-https');

/**
 * Expose `NielsenDCR` integration.
 */

var NielsenDCR = module.exports = integration('Nielsen DCR')
  .option('appId', '')
  .option('instanceName', '') // the snippet lets you override the instance so make sure you don't have any global window props w same value as this setting unless you are intentionally doing that.
  .tag('http:', '<script src="http://cdn-gl.imrworldwide.com/conf/{{ appId }}.js#name={{ instanceName }}&ns=NOLBUNDLE">')
  .tag('https:', '<script src="https://cdn-gl.imrworldwide.com/conf/{{ appId }}.js#name={{ instanceName }}&ns=NOLBUNDLE">');

/**
 * Initialize.
 * https://engineeringforum.nielsen.com/sdk/developers/product-dcr-video-implementation-bbsdk.php
 * @api public
 */

NielsenDCR.prototype.initialize = function() {
  /* eslint-disable */
  !function(t,n)
  {
  t[n]=t[n]||
  {
    nlsQ:function(e,o,c,r,s,i)
    {
     return t[n][o]=t[n][o]||{g:c||{},
     ggPM:function(e,c,r,s,i){(t[n][o].q=t[n][o].q||[]).push([e,c,r,s,i])}},t[n][o]
    }
  }
}
  (window,"NOLBUNDLE");
  /* eslint-enable */
  var protocol = useHttps() ? 'https:' : 'http:';
  var config = {};
  // debug mode
  if (!this.options.sfCode) config.nol_sdkDebug = 'debug';
  this._client = window.NOLBUNDLE.nlsQ(this.options.appId, this.options.instanceName, config);
  // we will need to keep our own state of the playhead position for the currently viewing ad or content
  this.counters = {
    ad: 0,
    content: 0
  };
  this.load(protocol, this.ready);
};

/**
 * Loaded?
 *
 * @api public
 * @return {boolean}
 */

NielsenDCR.prototype.loaded = function() {
  return this._client && typeof this._client.ggPM === 'function';
};

/**
 * Nielsen Heartbeat Counter
 * We need to send the playhead position every 1 second
 */

NielsenDCR.prototype.heartbeat = function(position, opts) {
  var self = this;
  var currentPosition = this.counters;
  opts = opts || {};
  // in case it is sent as a string
  if (typeof position !== 'number') position = parseInt(position, 10);

  // if position is passed in we should override the state of the counter with the explicit position given from the customer
  // segment expects our own heartbeats every 10 seconds, so we're adding 5 seconds of potential redundancy for buffer
  // for a total of 15 heartbeats
  currentPosition[opts.type] = position || currentPosition[opts.type];
  var limit = currentPosition[opts.type] + 15;

  this.heartbeatId = setInterval(function() {
    // if livestream, you need to send current UTC timestamp
    if (opts.type === 'content' && opts.livestream) {
      position = +Date.now(opts.timestamp);
      self._client.ggPM('setPlayheadPosition', position);
    } else if (position < limit) {
      self._client.ggPM('setPlayheadPosition', currentPosition[opts.type]);
      currentPosition[opts.type]++;
    }
  }, 1000);
};

/**
 * Video Content Started
 *
 * @api public
 */

NielsenDCR.prototype.videoContentStarted = function(track) {
  // clear the hearbeat in case there was pre-roll ads playing
  clearInterval(this.heartbeatId);
  var contentMetadata = {
    type: 'content',
    assetid: track.proxy('properties.asset_id'),
    program: track.proxy('properties.program'),
    title: track.proxy('properties.title'),
    length: track.proxy('properties.total_length'),
    isfullepisode: track.proxy('properties.full_episode') ? 'y' : 'n',
    mediaURL: track.proxy('context.page.url'),
    adloadtype: track.options(this.name).ad_load_type === 'dynamic' ? '2' : '1', // or linear
    hasAds: '2'
  };
  this._client.ggPM('loadMetadata', contentMetadata);
  this.heartbeat(track.proxy('properties.position'), {
    livestream: track.proxy('properties.livestream'),
    type: 'content',
    timestamp: track.timestamp()
  });
};

/**
 * Video Content Playing
 *
 * @api public
 */

NielsenDCR.prototype.videoContentPlaying = function(track) {
  // clear the background heartbeat since we will restart it
  clearInterval(this.heartbeatId);
  this.heartbeat(track.proxy('properties.position'), {
    livestream: track.proxy('properties.livestream'),
    type: 'content',
    timestamp: track.timestamp()
  });
};

/**
 * Video Content Completed
 *
 * @api public
 */

NielsenDCR.prototype.videoContentCompleted = function(track) {
  // immediately kill the background heartbeat
  clearInterval(this.heartbeatId);
  // for livestream just send the current utc timestamp
  var position;
  var timestamp = track.timestamp();
  if (track.proxy('properties.livestream')) position = +Date.now(timestamp);
  else position = track.proxy('properties.position');
  this._client.ggPM('setPlayheadPosition', position);
  this._client.ggPM('end', position);
};

/**
 * Video Ad Started
 *
 * @api public
 */

NielsenDCR.prototype.videoAdStarted = function(track) {
  // clear the hearbeat in case there was pre-roll ads playing
  clearInterval(this.heartbeatId);
  // edge case: if pre-roll, you must load the content metadata first
  // because nielsen ties ad attribution to the content not playback session
  var type = track.proxy('properties.type');
  if (typeof type === 'string') type = type.replace('-', '');
  if (type === 'preroll') {
    var contentMetadata = {
      type: 'content',
      assetid: track.proxy('properties.content.asset_id'),
      program: track.proxy('properties.content.program'),
      title: track.proxy('properties.content.title'),
      length: track.proxy('properties.content.total_length'),
      isfullepisode: track.proxy('properties.content.full_episode') ? 'y' : 'n',
      mediaURL: track.proxy('context.page.url'),
      adloadtype: track.proxy('properties.load_type') === 'dynamic' ? '2' : '1', // or linear
      hasAds: '2'
    };
    this._client.ggPM('loadMetadata', contentMetadata);
  }
  var adMetadata = {
    assetid: track.proxy('properties.asset_id'),
    type: type
  };

  this._client.ggPM('loadMetadata', adMetadata);
  this.heartbeat(track.proxy('properties.position'), { type: 'ad' });
};

/**
 * Video Ad Playing
 *
 * @api public
 */

NielsenDCR.prototype.videoAdPlaying = function(track) {
  // clear the background heartbeat since we will restart it
  clearInterval(this.heartbeatId);
  this.heartbeat(track.proxy('properties.position'), { type: 'ad' });
};

/**
 * Video Ad Completed
 *
 * @api public
 */

NielsenDCR.prototype.videoAdCompleted = function(track) {
  // immediately kill the background heartbeat
  clearInterval(this.heartbeatId);

  var position = track.proxy('properties.position');
  this._client.ggPM('setPlayheadPosition', position);
  this._client.ggPM('stop', position);
};

/**
 * Video Playback Interrupted
 *
 * @api public
 */

NielsenDCR.prototype.videoPlaybackInterrupted = function(track) {
  // immediately kill the background heartbeat
  clearInterval(this.heartbeatId);
  // if playback was interrupted during an ad, we only call `stop`
  // if interrupted during content play, we call both `end` and `stop`
  // if interrupted during ad or content, the defition of content/ad_asset_id that is sent should be mutually exclusive
  var position = track.proxy('properties.position');
  if (track.proxy('properties.ad_asset_id')) return this._client.ggPM('stop', position);
  this._client.ggPM('end', position);
  this._client.ggPM('stop', position);
};

/**
 * Video Playback Paused
 *
 * @api public
 */

NielsenDCR.prototype.videoPlaybackPaused = function() {
  // pausing the heartbeats constitutes a pause in nielsen
  clearInterval(this.heartbeatId);
};

/**
 * Video Playback Resumed
 *
 * @api public
 */

NielsenDCR.prototype.videoPlaybackResumed = function(track) {
  clearInterval(this.heatbeatId);
  // reset the counter to the current position from this event
  // and continue the counter
  var contentAssetId = track.proxy('properties.content_asset_id');
  // if properly implemented, the point in which the playback is resumed
  // you should _only_ be sending the asset_id of whatever you are resuming in: content or ad
  var type = contentAssetId ? 'content' : 'ad';
  this.heartbeat(track.proxy('properties.position'), { type: type });
};

/**
 * Video Playback Completed
 *
 * @api public
 */

NielsenDCR.prototype.videoPlaybackCompleted = function(track) {
  clearInterval(this.hearbeatId);
  // for livestream just send the current utc timestamp
  var position;
  if (track.proxy('properties.livestream')) position = +Date.now(track.timestamp());
  else position = track.proxy('properties.position');
  this._client.ggPM('setPlayheadPosition', position);
  this._client.ggPM('stop', position);
};

==> analytics.js-integration-nielsen-dcr-0.0.4.tgz: package/lib/index.js <==
'use strict';

/**
 * Module dependencies.
 */

var integration = require('@segment/analytics.js-integration');
var useHttps = require('use-https');

/**
 * Expose `NielsenDCR` integration.
 */

var NielsenDCR = module.exports = integration('Nielsen DCR')
  .option('appId', '')
  .option('instanceName', '') // the snippet lets you override the instance so make sure you don't have any global window props w same value as this setting unless you are intentionally doing that.
  .tag('http:', '<script src="http://cdn-gl.imrworldwide.com/conf/{{ appId }}.js#name={{ instanceName }}&ns=NOLBUNDLE">')
  .tag('https:', '<script src="https://cdn-gl.imrworldwide.com/conf/{{ appId }}.js#name={{ instanceName }}&ns=NOLBUNDLE">');

/**
 * Initialize.
 * https://engineeringforum.nielsen.com/sdk/developers/product-dcr-video-implementation-bbsdk.php
 * @api public
 */

NielsenDCR.prototype.initialize = function() {
  /* eslint-disable */
  !function(t,n)
  {
  t[n]=t[n]||
  {
    nlsQ:function(e,o,c,r,s,i)
    {
     return t[n][o]=t[n][o]||{g:c||{},
     ggPM:function(e,c,r,s,i){(t[n][o].q=t[n][o].q||[]).push([e,c,r,s,i])}},t[n][o]
    }
  }
}
  (window,"NOLBUNDLE");
  /* eslint-enable */
  var protocol = useHttps() ? 'https:' : 'http:';
  var config = {};
  // debug mode
  if (!this.options.sfCode) config.nol_sdkDebug = 'debug';
  this._client = window.NOLBUNDLE.nlsQ(this.options.appId, this.options.instanceName, config);
  // we will need to keep our own state of the playhead position for the currently viewing ad or content
  this.counters = {
    ad: 0,
    content: 0
  };
  this.load(protocol, this.ready);
};

/**
 * Loaded?
 *
 * @api public
 * @return {boolean}
 */

NielsenDCR.prototype.loaded = function() {
  return this._client && typeof this._client.ggPM === 'function';
};

/**
 * Nielsen Heartbeat Counter
 * We need to send the playhead position every 1 second
 */

NielsenDCR.prototype.heartbeat = function(position, opts) {
  var self = this;
  var currentPosition = this.counters;
  opts = opts || {};
  // in case it is sent as a string
  if (typeof position !== 'number') position = parseInt(position, 10);

  // if position is passed in we should override the state of the counter with the explicit position given from the customer
  // segment expects our own heartbeats every 10 seconds, so we're adding 5 seconds of potential redundancy for buffer
  // for a total of 15 heartbeats
  currentPosition[opts.type] = position || currentPosition[opts.type];
  var limit = currentPosition[opts.type] + 15;

  this.heartbeatId = setInterval(function() {
    // if livestream, you need to send current UTC timestamp
    if (opts.type === 'content' && opts.livestream) {
      currentPosition.content = new Date(opts.timestamp);
      var currentTime = currentPosition.content;
      // for livestream, properties.position is a negative integer representing offset in seconds from current time
      currentTime.setSeconds(currentTime.getSeconds() + position);
      self._client.ggPM('setPlayheadPosition', +Date.now(currentTime)); // UTC
      // increment timestamp by 1 second
      currentTime.setSeconds(currentTime.getSeconds() + 1);
    } else if (position < limit) {
      self._client.ggPM('setPlayheadPosition', currentPosition[opts.type]);
      // increment playhead by 1 second
      currentPosition[opts.type]++;
    }
  }, 1000);
};

/**
 * Video Content Started
 *
 * @api public
 */

NielsenDCR.prototype.videoContentStarted = function(track) {
  // clear the hearbeat in case there was pre-roll ads playing
  clearInterval(this.heartbeatId);
  var contentMetadata = {
    type: 'content',
    assetid: track.proxy('properties.asset_id'),
    program: track.proxy('properties.program'),
    title: track.proxy('properties.title'),
    // hardcode 86400 if livestream ¯\_(ツ)_/¯
    length: track.proxy('properties.livestream') ? 86400 : track.proxy('properties.total_length'),
    isfullepisode: track.proxy('properties.full_episode') ? 'y' : 'n',
    mediaURL: track.proxy('context.page.url'),
    adloadtype: track.options(this.name).ad_load_type === 'dynamic' ? '2' : '1', // or linear
    hasAds: '2'
  };
  this._client.ggPM('loadMetadata', contentMetadata);
  this.heartbeat(track.proxy('properties.position'), {
    livestream: track.proxy('properties.livestream'),
    type: 'content',
    timestamp: track.timestamp()
  });
};

/**
 * Video Content Playing
 *
 * @api public
 */

NielsenDCR.prototype.videoContentPlaying = function(track) {
  // clear the background heartbeat since we will restart it
  clearInterval(this.heartbeatId);
  this.heartbeat(track.proxy('properties.position'), {
    livestream: track.proxy('properties.livestream'),
    type: 'content',
    timestamp: track.timestamp()
  });
};

/**
 * Video Content Completed
 *
 * @api public
 */

NielsenDCR.prototype.videoContentCompleted = function(track) {
  // immediately kill the background heartbeat
  clearInterval(this.heartbeatId);
  // for livestream just send the current utc timestamp
  var position;
  var timestamp = track.timestamp();
  if (track.proxy('properties.livestream')) position = +Date.now(timestamp);
  else position = track.proxy('properties.position');
  this._client.ggPM('setPlayheadPosition', position);
  this._client.ggPM('end', position);
};

/**
 * Video Ad Started
 *
 * @api public
 */

NielsenDCR.prototype.videoAdStarted = function(track) {
  // clear the hearbeat in case there was pre-roll ads playing
  clearInterval(this.heartbeatId);
  // edge case: if pre-roll, you must load the content metadata first
  // because nielsen ties ad attribution to the content not playback session
  var type = track.proxy('properties.type');
  if (typeof type === 'string') type = type.replace('-', '');
  if (type === 'preroll') {
    var contentMetadata = {
      type: 'content',
      assetid: track.proxy('properties.content.asset_id'),
      program: track.proxy('properties.content.program'),
      title: track.proxy('properties.content.title'),
      length: track.proxy('properties.content.total_length'),
      isfullepisode: track.proxy('properties.content.full_episode') ? 'y' : 'n',
      mediaURL: track.proxy('context.page.url'),
      adloadtype: track.proxy('properties.load_type') === 'dynamic' ? '2' : '1', // or linear
      hasAds: '2'
    };
    this._client.ggPM('loadMetadata', contentMetadata);
  }
  var adMetadata = {
    assetid: track.proxy('properties.asset_id'),
    type: type
  };

  this._client.ggPM('loadMetadata', adMetadata);
  this.heartbeat(track.proxy('properties.position'), { type: 'ad' });
};

/**
 * Video Ad Playing
 *
 * @api public
 */

NielsenDCR.prototype.videoAdPlaying = function(track) {
  // clear the background heartbeat since we will restart it
  clearInterval(this.heartbeatId);
  this.heartbeat(track.proxy('properties.position'), { type: 'ad' });
};

/**
 * Video Ad Completed
 *
 * @api public
 */

NielsenDCR.prototype.videoAdCompleted = function(track) {
  // immediately kill the background heartbeat
  clearInterval(this.heartbeatId);

  var position = track.proxy('properties.position');
  this._client.ggPM('setPlayheadPosition', position);
  this._client.ggPM('stop', position);
};

/**
 * Video Playback Interrupted
 *
 * @api public
 */

NielsenDCR.prototype.videoPlaybackInterrupted = function(track) {
  // immediately kill the background heartbeat
  clearInterval(this.heartbeatId);
  // if playback was interrupted during an ad, we only call `stop`
  // if interrupted during content play, we call both `end` and `stop`
  // if interrupted during ad or content, the defition of content/ad_asset_id that is sent should be mutually exclusive
  var position = track.proxy('properties.position');
  if (track.proxy('properties.ad_asset_id')) return this._client.ggPM('stop', position);
  this._client.ggPM('end', position);
  this._client.ggPM('stop', position);
};

/**
 * Video Playback Paused
 *
 * @api public
 */

NielsenDCR.prototype.videoPlaybackPaused = function(track) {
  // pausing the heartbeats constitutes a pause in nielsen
  clearInterval(this.heartbeatId);
  this._client.ggPM('stop', track.proxy('properties.position'));
};

/**
 * Video Playback Resumed
 *
 * @api public
 */

NielsenDCR.prototype.videoPlaybackResumed = function(track) {
  clearInterval(this.heatbeatId);
  // reset the counter to the current position from this event
  // and continue the counter
  var contentAssetId = track.proxy('properties.content_asset_id');
  // if properly implemented, the point in which the playback is resumed
  // you should _only_ be sending the asset_id of whatever you are resuming in: content or ad
  var type = contentAssetId ? 'content' : 'ad';
  this.heartbeat(track.proxy('properties.position'), { type: type });
};

/**
 * Video Playback Completed
 *
 * @api public
 */

NielsenDCR.prototype.videoPlaybackCompleted = function(track) {
  clearInterval(this.hearbeatId);
  // for livestream just send the current utc timestamp
  var position;
  if (track.proxy('properties.livestream')) position = +Date.now(track.timestamp());
  else position = track.proxy('properties.position');
  this._client.ggPM('setPlayheadPosition', position);
  this._client.ggPM('stop', position);
};
